package com.akamai.osmf.samples.presenter
{
	import com.akamai.osmf.samples.view.BaseVideoView;
	
	import flash.display.StageDisplayState;
	import flash.events.EventDispatcher;
	import flash.events.FullScreenEvent;
	import flash.geom.Rectangle;
	import flash.media.SoundTransform;
	import flash.net.SharedObject;
	import flash.system.Capabilities;
	
	import org.openvideoplayer.components.ui.controlbar.ControlBar;
	import org.openvideoplayer.components.ui.controlbar.view.DualTimeCodeDisplay;
	import org.openvideoplayer.components.ui.controlbar.view.FullscreenButton;
	import org.openvideoplayer.components.ui.controlbar.view.PlayPauseButton;
	import org.openvideoplayer.components.ui.controlbar.view.ScrubBar;
	import org.openvideoplayer.components.ui.controlbar.view.VolumeControl;
	import org.openvideoplayer.components.ui.posterframe.PosterFramePlayButton;
	import org.openvideoplayer.components.ui.shared.ControlType;
	import org.openvideoplayer.components.ui.shared.event.ControlEvent;
	import org.openvideoplayer.components.ui.spinner.SpinnerView;
	import org.osmf.events.BufferEvent;
	import org.osmf.events.DVREvent;
	import org.osmf.events.MediaElementEvent;
	import org.osmf.events.PlayEvent;
	import org.osmf.events.TimeEvent;
	import org.osmf.media.MediaElement;
	import org.osmf.media.MediaFactory;
	import org.osmf.media.MediaPlayer;
	import org.osmf.net.StreamType;
	import org.osmf.net.StreamingURLResource;
	import org.osmf.traits.DVRTrait;
	import org.osmf.traits.MediaTraitBase;
	import org.osmf.traits.MediaTraitType;
	import org.osmf.traits.PlayState;
	import org.osmf.utils.TimeUtil;
	
	public class BaseControlBarPresenter extends EventDispatcher
	{
		private var sharedObject:SharedObject;
		private var view:BaseVideoView;
		private var mediaResource:String;
		private var playPauseButton:PlayPauseButton;
		private var dualTimeCodeDisplay:DualTimeCodeDisplay;
		private var volumeControl:VolumeControl;
		private var fullscreenButton:FullscreenButton;
		private var posterFrameButton:PosterFramePlayButton;
		private var duration:Number = 0;
		private var seeking:Boolean = false;
		private var isLive:Boolean = false;
		private var originalVideoRect:Rectangle;
		
		private var _mediaPlayer:MediaPlayer;
		private var _mediaFactory:MediaFactory;
		private var _controlBar:ControlBar;
		private var _scrubBar:ScrubBar;
		private var _spinnerView:SpinnerView;
		
		
		public function BaseControlBarPresenter(view:BaseVideoView, mediaPlayer:MediaPlayer, mediaFactory:MediaFactory, mediaResource:String)
		{
			super();
			
			this.view = view;
			_mediaPlayer = mediaPlayer;
			_mediaFactory = mediaFactory;
			this.mediaResource = mediaResource;
			_controlBar = view.controlBar;
			posterFrameButton = view.posterFrameButton;
			_spinnerView = view.spinnerView;
			
			setupMediaPlayer();
			
			setControlBarReferences();
			setupControlBarListeners();
			
			try
			{
				sharedObject = SharedObject.getLocal("AkamaiControlBarProperties");
			}
			catch (e:Error)
			{
				// We'll fail silently on this but log it
				debug("Cannot load or create shared object: "+e.toString());
			}
			
			initVolume();
			
		}
		
		public function get mediaFactory():MediaFactory
		{
			return _mediaFactory;
		}
		
		public function get scrubBar():ScrubBar
		{
			return _scrubBar;
		}
		
		public function get controlBar():ControlBar
		{
			return _controlBar;
		}
		
		protected function setupMediaPlayer():void
		{
			_mediaPlayer.autoPlay = false;
			setupMediaPlayerListeners();
			_mediaPlayer.media = _mediaFactory.createMediaElement(new StreamingURLResource(mediaResource, StreamType.LIVE_OR_RECORDED));
			setupMediaListeners();
		}
		
		public function destroy():void
		{
			setupMediaPlayerListeners(false);
			view = null;
			mediaResource = null;
			setupControlBarListeners(false);
			_controlBar = null;
		}
		
		public function get mediaPlayer():MediaPlayer
		{
			return _mediaPlayer;
		}
		
		protected function setControlBarReferences():void
		{
			
			playPauseButton = _controlBar.getControl(ControlType.PLAY_PAUSE_BUTTON) as PlayPauseButton;
			_scrubBar = _controlBar.getControl(ControlType.SCRUB_BAR) as ScrubBar;
			dualTimeCodeDisplay = _controlBar.getControl(ControlType.CURRENT_AND_DURATION_TIME_DISPLAY) as DualTimeCodeDisplay;
			volumeControl = _controlBar.getControl(ControlType.VOLUME_CONTROL) as VolumeControl;
			fullscreenButton = _controlBar.getControl(ControlType.FULLSCREEN_BUTTON) as FullscreenButton;
		}
		
		protected function onPlayClick(event:ControlEvent):void
		{
			if (_mediaPlayer.playing && _mediaPlayer.canPause)
			{
				_mediaPlayer.pause();
			}
			else if (!_mediaPlayer.playing && _mediaPlayer.canPlay)
			{
				_mediaPlayer.play();
			}
			
			showPosterFrameButton(false);
		}
		
		protected function onSeekBegin(event:ControlEvent):void
		{
			seeking = true;
		}
		
		protected function onSeekComplete(event:ControlEvent):void
		{
			_mediaPlayer.seek(_scrubBar.getThumbPosition(_mediaPlayer.duration));
			seeking = false;
		}
		
		protected function onVolumeChange(event:ControlEvent):void
		{
			_mediaPlayer.volume = event.data.value;
			
			if (sharedObject != null)
			{
				sharedObject.data.volume = event.data.value;
			}
		}
		
		protected function onFullscreenClick(event:ControlEvent):void
		{
			
			if (originalVideoRect == null)
			{
				originalVideoRect 	= new Rectangle();
				originalVideoRect.x = view.mediaPlayerSprite.x;
				originalVideoRect.y = view.mediaPlayerSprite.y;
				originalVideoRect.width = view.mediaPlayerSprite.width;
				originalVideoRect.height = view.mediaPlayerSprite.height;
			}
			
			
			if (view.stage.displayState == StageDisplayState.NORMAL)
			{
				var w:Number;
				var h:Number;
				
				
				if (Capabilities.screenResolutionX/ Capabilities.screenResolutionY> view.mediaPlayerSprite.width / view.mediaPlayerSprite.height)
				{
					h = Capabilities.screenResolutionY;
					w = view.mediaPlayerSprite.width * h / view.mediaPlayerSprite.height;
				}
				else
				{
					w = Capabilities.screenResolutionX;
					h = view.mediaPlayerSprite.height * w / view.mediaPlayerSprite.width;
				} 
				
				view.mediaPlayerSprite.width = Math.max(w, view.mediaPlayerSprite.width);
				view.mediaPlayerSprite.height = Math.max(h, view.mediaPlayerSprite.height);
				view.stage.fullScreenSourceRect = new Rectangle(view.mediaPlayerSprite.x, view.mediaPlayerSprite.y, w, h);
			}
			
			view.stage.displayState = view.stage.displayState == StageDisplayState.NORMAL ? StageDisplayState.FULL_SCREEN : StageDisplayState.NORMAL;	
			
			// hide the controlBar in fullscreen mode. 
			_controlBar.visible = false;
		}
		
		protected function onFullScreenEvent(event:FullScreenEvent):void
		{
			if (view.stage.displayState == StageDisplayState.NORMAL)
			{
				fullscreenButton.currentState = ControlEvent.FULLSCREEN;
				view.mediaPlayerSprite.x = originalVideoRect.x;
				view.mediaPlayerSprite.y = originalVideoRect.y;
				view.mediaPlayerSprite.width = originalVideoRect.width;
				view.mediaPlayerSprite.height = originalVideoRect.height;
				_controlBar.visible = true;
			}
		}
		
		protected function onPosterFrameClick(event:ControlEvent):void
		{
			onPlayClick(null);
			playPauseButton.currentState = ControlEvent.PLAY;
		}
		
		protected function showPosterFrameButton(value:Boolean = true):void
		{
			if (posterFrameButton != null)
			{
				posterFrameButton.visible = value;
			}
		}
		
		protected function initVolume():void
		{
			debug("BaseControlBarPresenter.initVolume()" + view.mediaPlayerSprite.mediaPlayer.volume);
			volumeControl.setThumbPosition(sharedObject != null && !isNaN(sharedObject.data.volume) ? sharedObject.data.volume : view.mediaPlayerSprite.mediaPlayer.volume, 1);
		}
		
		private function setVolumeOnNetStream(value:Number):void
		{
			var soundTransform:SoundTransform = new SoundTransform(value);
		}
		
		protected function setupControlBarListeners(add:Boolean = true):void
		{
			if (add)
			{
				_controlBar.addEventListener(ControlEvent.PLAY, onPlayClick);
				_controlBar.addEventListener(ControlEvent.PAUSE, onPlayClick);
				_controlBar.addEventListener(ControlEvent.SEEK_BEGIN, onSeekBegin);
				_controlBar.addEventListener(ControlEvent.SEEK_COMPLETE, onSeekComplete);
				_controlBar.addEventListener(ControlEvent.VOLUME_CHANGE, onVolumeChange);
				_controlBar.addEventListener(ControlEvent.FULLSCREEN, onFullscreenClick);
				_controlBar.addEventListener(ControlEvent.NORMALSCREEN, onFullscreenClick);
				_controlBar.addEventListener(ControlEvent.SEEK_CHANGE, onSeekChange);
				_controlBar.stage.addEventListener(FullScreenEvent.FULL_SCREEN, onFullScreenEvent);
				
				
				if (posterFrameButton)
				{
					posterFrameButton.addEventListener(ControlEvent.PLAY, onPosterFrameClick);
				}
			}
			else
			{
				_controlBar.removeEventListener(ControlEvent.PLAY, onPlayClick);
				_controlBar.removeEventListener(ControlEvent.PAUSE, onPlayClick);
				_controlBar.removeEventListener(ControlEvent.SEEK_BEGIN, onSeekBegin);
				_controlBar.removeEventListener(ControlEvent.SEEK_COMPLETE, onSeekComplete);
				_controlBar.removeEventListener(ControlEvent.VOLUME_CHANGE, onVolumeChange);
				_controlBar.removeEventListener(ControlEvent.FULLSCREEN, onFullscreenClick);
				_controlBar.removeEventListener(ControlEvent.SEEK_CHANGE, onSeekChange);
				_controlBar.removeEventListener(ControlEvent.NORMALSCREEN, onFullscreenClick);
				
				if (posterFrameButton)
				{
					posterFrameButton.removeEventListener(ControlEvent.PLAY, onPosterFrameClick);
				}
			}
		}
		
		protected function setupMediaPlayerListeners(add:Boolean = true):void
		{
			if (add)
			{
				_mediaPlayer.addEventListener(TimeEvent.CURRENT_TIME_CHANGE, onCurrentTimeChange);
				_mediaPlayer.addEventListener(TimeEvent.DURATION_CHANGE, onDurationChange);
				_mediaPlayer.addEventListener(TimeEvent.COMPLETE, onComplete);
				_mediaPlayer.addEventListener(PlayEvent.PLAY_STATE_CHANGE, onPlayStateChange);
				_mediaPlayer.addEventListener(BufferEvent.BUFFERING_CHANGE, onBufferingChange);
			}
			else
			{
				_mediaPlayer.removeEventListener(TimeEvent.CURRENT_TIME_CHANGE, onCurrentTimeChange);
				_mediaPlayer.removeEventListener(TimeEvent.DURATION_CHANGE, onDurationChange);
				_mediaPlayer.removeEventListener(TimeEvent.COMPLETE, onComplete);
				_mediaPlayer.removeEventListener(PlayEvent.PLAY_STATE_CHANGE, onPlayStateChange);
				_mediaPlayer.removeEventListener(BufferEvent.BUFFERING_CHANGE, onBufferingChange);
			}
		}
		
		protected function onSeekChange(event:ControlEvent):void
		{
			if (!isLive)
			{
				updateTime(_scrubBar.getThumbPosition(_mediaPlayer.duration),duration);
			}
		}
		
		private function setupMediaListeners(add:Boolean = true):void
		{
			if (add)
			{
				_mediaPlayer.media.addEventListener(MediaElementEvent.TRAIT_ADD, onTraitAdd);
				_mediaPlayer.media.addEventListener(MediaElementEvent.TRAIT_REMOVE, onTraitRemove);
			}
			else
			{
				_mediaPlayer.media.removeEventListener(MediaElementEvent.TRAIT_ADD, onTraitAdd);
				_mediaPlayer.media.removeEventListener(MediaElementEvent.TRAIT_REMOVE, onTraitRemove);
			}
		}

		private function onPlayStateChange(event:PlayEvent):void
		{
			switch (event.playState)
			{
				case PlayState.PLAYING:
					playPauseButton.currentState =  ControlEvent.PLAY;
					break;
				case PlayState.STOPPED:
				case PlayState.PAUSED:
					playPauseButton.currentState = ControlEvent.PAUSE;
					break;
			}
		}
		
		private function onBufferingChange(event:BufferEvent):void
		{
			handleBuffering(event.buffering);
		}
		
		
		private function onTraitAdd(event:MediaElementEvent):void
		{
			var trait:MediaTraitBase = (event.target as MediaElement).getTrait(event.traitType);
			
			switch (event.traitType)
			{
				case MediaTraitType.DVR:
					var dvrTrait:DVRTrait = (event.target as MediaElement).getTrait(MediaTraitType.DVR) as DVRTrait;
					isLive = dvrTrait.isRecording;
					dvrTrait.addEventListener(DVREvent.IS_RECORDING_CHANGE, onIsRecordingChange);
					
					if (!isNaN(dvrTrait.windowDuration))
					{
						debug("DVR Window="+TimeUtil.formatAsTimeCode(dvrTrait.windowDuration));
					}
					break;
			}
		}
		
		private function onTraitRemove(event:MediaElementEvent):void
		{
			var trait:MediaTraitBase = (event.target as MediaElement).getTrait(event.traitType);
			
			switch (event.traitType)
			{
				case MediaTraitType.DVR:
					var dvrTrait:DVRTrait = (event.target as MediaElement).getTrait(MediaTraitType.DVR) as DVRTrait;
					dvrTrait.removeEventListener(DVREvent.IS_RECORDING_CHANGE, onIsRecordingChange);
					break;
			}
		}
		
		private function onIsRecordingChange(event:DVREvent):void
		{
			var dvrTrait:DVRTrait = _mediaPlayer.media.getTrait(MediaTraitType.DVR) as DVRTrait;
			isLive = (dvrTrait != null) ? isLive = dvrTrait.isRecording : false;	
		}
		
		private function handleBuffering(buffering:Boolean):void
		{				
			_spinnerView.visible = buffering;
		}
				
		private function onCurrentTimeChange(event:TimeEvent):void
		{
			if (duration > 0 && !seeking)
			{
				_scrubBar.setThumbPosition(event.time, duration);
				updateTime(event.time, duration);
			}
		}
		
		private function updateTime(eventTime:Number,duration:Number):void 
		{
			dualTimeCodeDisplay.setTimeCodeText(TimeUtil.formatAsTimeCode(eventTime), TimeUtil.formatAsTimeCode(duration));	
		}
		
		private function onDurationChange(event:TimeEvent):void
		{
			var durationIsValid:Boolean = !isNaN(event.time) && (event.time > 0);
			
			if (durationIsValid && _mediaPlayer.temporal && !seeking)
			{
				duration = event.time;
				_scrubBar.setThumbPosition(_mediaPlayer.currentTime, duration);
				updateTime(_mediaPlayer.currentTime, duration);
			}
		}
		
		private function onComplete(event:TimeEvent):void
		{
			debug("onComplete");
			_spinnerView.visible = false;
		}
		
		private function debug(msg:String):void
		{
			trace(msg);
		}
	}
}
